-- PC6001 top module
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity RGBCONV is
	port (
		DRAM_D			: inout std_logic_vector(15 downto 0);	-- SDRAM data bus
		DRAM_A			: out   std_logic_vector(12 downto 0);	-- SDRAM address bus
		DRAM_CLK		: out   std_logic;						-- SDRAM clock output
		DRAM_CKE		: out   std_logic;						-- SDRAM clock enable
		DRAM_LDQM		: out   std_logic;						-- SDRAM LowerByte Data Mask
		DRAM_UDQM		: out   std_logic;						-- SDRAM UpperByte Data Mask
		DRAM_WE_N		: out   std_logic;						-- SDRAM write Enable
		DRAM_CAS_N		: out   std_logic;						-- SDRAM CAS
		DRAM_RAS_N		: out   std_logic;						-- SDRAM RAS
		DRAM_CS_N		: out   std_logic;						-- SDRAM chip select
		DRAM_BA_1		: out   std_logic;						-- SDRAM Bank #1
		DRAM_BA_0		: out   std_logic;						-- SDRAM Bank #0
		FLASH_D15_AM1	: inout std_logic;						-- FLASH data bus bit15 or adrress-1
		FLASH_D			: inout std_logic_vector(14 downto 0);	-- FLASH data bus
		FLASH_A			: out   std_logic_vector(21 downto 0);	-- FLASH adrress bus
		FLASH_WE_N		: out   std_logic;						-- FLASH write enable
		FLASH_RESET_N	: out   std_logic;						-- FLASH reset
		FLASH_WP_N		: out   std_logic;						-- FLASH write protect
		FLASH_RY		: in    std_logic;						-- FLASH ready
		FLASH_CE_N		: out   std_logic;						-- FLASH chip enable
		FLASH_OE_N		: out   std_logic;						-- FLASH output enable
		FLASH_BYTE_N	: out   std_logic;						-- FLASH byte mode
		VGA_R			: out   std_logic_vector(3 downto 0);	-- VGA red data
		VGA_G			: out   std_logic_vector(3 downto 0);	-- VGA green data
		VGA_B			: out   std_logic_vector(3 downto 0);	-- VGA blue data
		VGA_HS			: out   std_logic;						-- VGA H_SYNC
		VGA_VS			: out   std_logic;						-- VGA V_SYNC
		HEX3_D			: out   std_logic_vector(6 downto 0);	-- 7segment #3
		HEX3_DP			: out   std_logic;						-- 7segment #3 DP
		HEX2_D			: out   std_logic_vector(6 downto 0);	-- 7segment #2
		HEX2_DP			: out   std_logic;						-- 7segment #2 DP
		HEX1_D			: out   std_logic_vector(6 downto 0);	-- 7segment #1
		HEX1_DP			: out   std_logic;						-- 7segment #1 DP
		HEX0_D			: out   std_logic_vector(6 downto 0);	-- 7segment #0
		HEX0_DP			: out   std_logic;						-- 7segment #0 DP
		LEDG			: out   std_logic_vector(9 downto 0);	-- LED
		LCD_D			: inout std_logic_vector(7 downto 0);	-- LCD data bus
		LCD_BLON		: out   std_logic;						-- LCD back light on
		LCD_RS			: out   std_logic;						-- LCD command/data select
		LCD_RW			: out   std_logic;						-- LCD read/write
		LCD_EN			: out   std_logic;						-- LCD enable
		CLK50M1			: in    std_logic;						-- clock 50MHz input #1
		CLK50M0			: in    std_logic;						-- clock 50MHz input #0
		UART_RXD		: in    std_logic;						-- UART Rx
		UART_RTS		: in    std_logic;						-- UART RTS
		UART_TXD		: out   std_logic;						-- UART Tx
		UART_CTS		: out   std_logic;						-- UART CTS
		PS2_KBDAT		: inout std_logic;						-- PS2 keyboard data
		PS2_KBCLK		: inout std_logic;						-- PS2 keyboard clock
		PS2_MSDAT		: inout std_logic;						-- PS2 mouse data
		PS2_MSCLK		: inout std_logic;						-- PS2 mouse clock
		BUTTON			: in    std_logic_vector(2 downto 0);	-- push button
		SW				: in    std_logic_vector(9 downto 0);	-- DIPSW
		GPIO1_D			: inout std_logic_vector(31 downto 0);	-- GPIO #1 data
		GPIO1_CLKIN		: in    std_logic_vector(1 downto 0);	-- GPIO #1 clock input
		GPIO1_CLKOUT	: out   std_logic_vector(1 downto 0);	-- GPIO #1 clock output
		GPIO0_D			: inout std_logic_vector(31 downto 0);	-- GPIO #0 data
		GPIO0_CLKIN		: in    std_logic_vector(1 downto 0);	-- GPIO #0 clock input
		GPIO0_CLKOUT	: out   std_logic_vector(1 downto 0);	-- GPIO #0 clock output
		SD_DAT			: inout std_logic_vector(3 downto 0);	-- SD card data
		SD_CMD			: inout std_logic;						-- SD card command
		SD_CLK			: out   std_logic;						-- SD card clock output
		SD_WP_N			: in    std_logic						-- SD card write protect
	);
end RGBCONV;

architecture RTL of RGBCONV is

	component CLKGEN is
		port (
			CLK50M		: in  std_logic;
			RSTN		: in  std_logic;
			CLK108MOUT	: out std_logic;
			LOCK_PLL	: out std_logic
		);
	end component;

	component P6SYNC is
		port (
			P6_HSYNCN	: in  std_logic;
			P6_VSYNCN	: in  std_logic;
			P6_RGBH		: in  std_logic_vector(3 downto 0);
			P6_CLK		: in  std_logic;
			CLK108M		: in  std_logic;
			RSTN		: in  std_logic;
			SRMODE		: out std_logic;
			CLKENB		: out std_logic;
			DT_HSYNCN	: out std_logic;
			DT_VSYNCN	: out std_logic;
			DT_RGBH		: out std_logic_vector(3 downto 0);
			SFTDT_U		: out std_logic_vector(2 downto 0);
			SFTDT_D		: out std_logic_vector(2 downto 0);
			SFTHS_U		: out std_logic_vector(2 downto 0);
			SFTHS_D		: out std_logic_vector(2 downto 0);
			LOCK1		: out std_logic;
			LOCK2		: out std_logic
		);
	end component;

	component DATASFT is
		port (
			HSYNCN		: in  std_logic;
			VSYNCN		: in  std_logic;
			RGBH		: in  std_logic_vector(3 downto 0);
			CLKENB		: in  std_logic;
			SRMODE		: in  std_logic;
			LOCK1		: in  std_logic;
			LOCK2		: in  std_logic;
			CLK108M		: in  std_logic;
			RSTN		: in  std_logic;
			HSYNCN_O	: out std_logic;
			VSYNCN_O	: out std_logic;
			RGBH_O		: out std_logic_vector(3 downto 0);
			LOCK3		: out std_logic;
			LOCK4		: out std_logic
		);
	end component;

	component VGAOUT is
		port (
			DISPD		: in  std_logic_vector(3 downto 0);
			DISPTMG_HS	: in  std_logic;
			DISPTMG_VS	: in  std_logic;
			CLKENB		: in  std_logic;
			SRMODE		: in  std_logic;
			SC4COLORON	: in  std_logic;
			SC4COLORMD	: in  std_logic_vector(1 downto 0);
			LOCK_PLL	: in  std_logic;
			CLK108M		: in  std_logic;
			RSTN		: in  std_logic;
			VGA_R		: out std_logic_vector(3 downto 0);
			VGA_G		: out std_logic_vector(3 downto 0);
			VGA_B		: out std_logic_vector(3 downto 0);
			VGA_HS		: out std_logic;
			VGA_VS		: out std_logic
		);
	end component;

	component COLSEL is
		port (
			MDSEL		: in  std_logic;
			LOCK_PLL	: in  std_logic;
			CLK108M		: in  std_logic;
			RSTN		: in  std_logic;
			COLORMD		: out std_logic_vector(2 downto 0)
		);
	end component;

	component SEG7DISP is
		port (
			SFTDT_U		: in  std_logic_vector(2 downto 0);
			SFTDT_D		: in  std_logic_vector(2 downto 0);
			SFTHS_U		: in  std_logic_vector(2 downto 0);
			SFTHS_D		: in  std_logic_vector(2 downto 0);
			HEX3_D		: out std_logic_vector(6 downto 0);
			HEX3_DP		: out std_logic;
			HEX2_D		: out std_logic_vector(6 downto 0);
			HEX2_DP		: out std_logic;
			HEX1_D		: out std_logic_vector(6 downto 0);
			HEX1_DP		: out std_logic;
			HEX0_D		: out std_logic_vector(6 downto 0);
			HEX0_DP		: out std_logic
		);
	end component;

-- PC-6001 input
	signal p6_r			: std_logic;
	signal p6_g			: std_logic;
	signal p6_b			: std_logic;
	signal p6_h			: std_logic;
	signal p6_hsyncn	: std_logic;
	signal p6_vsyncn	: std_logic;
	signal p6_rgbh		: std_logic_vector(3 downto 0);
	signal p6_clk		: std_logic;

-- clkgen
	signal rstn			: std_logic;
	signal clk108m		: std_logic;
	signal lock_pll		: std_logic;

-- p6sync
	signal sy_hsyncn	: std_logic;
	signal sy_vsyncn	: std_logic;
	signal sy_srmode	: std_logic;
	signal sy_clkenb	: std_logic;
	signal sy_rgbh		: std_logic_vector(3 downto 0);
	signal sy_sftdt_u	: std_logic_vector(2 downto 0);
	signal sy_sftdt_d	: std_logic_vector(2 downto 0);
	signal sy_sfths_u	: std_logic_vector(2 downto 0);
	signal sy_sfths_d	: std_logic_vector(2 downto 0);
	signal sy_lock1		: std_logic;
	signal sy_lock2		: std_logic;

-- datasft
	signal dt_hsyncn	: std_logic;
	signal dt_vsyncn	: std_logic;
	signal dt_rgbh		: std_logic_vector(3 downto 0);
	signal dt_lock3		: std_logic;
	signal dt_lock4		: std_logic;

-- vgaout
	signal vga_r_i		: std_logic_vector(3 downto 0);
	signal vga_g_i		: std_logic_vector(3 downto 0);
	signal vga_b_i		: std_logic_vector(3 downto 0);
	signal vga_hs_i		: std_logic;
	signal vga_vs_i		: std_logic;

-- colormd
	signal mdsel		: std_logic;
	signal colormd		: std_logic_vector(2 downto 0);

-- seg7disp
	signal sg_hex3_d	: std_logic_vector(6 downto 0);
	signal sg_hex3_dp	: std_logic;
	signal sg_hex2_d	: std_logic_vector(6 downto 0);
	signal sg_hex2_dp	: std_logic;
	signal sg_hex1_d	: std_logic_vector(6 downto 0);
	signal sg_hex1_dp	: std_logic;
	signal sg_hex0_d	: std_logic_vector(6 downto 0);
	signal sg_hex0_dp	: std_logic;

-- prob
	signal probdat1		: std_logic_vector(7 downto 0);
	signal probdat2		: std_logic_vector(15 downto 0);
	signal probdat3		: std_logic_vector(7 downto 0);

	signal gpio0_d_tmp		: std_logic_vector(31 downto 0);
	signal gpio0_clkout_tmp	: std_logic_vector(1 downto 0);

	signal gpio1_d_tmp		: std_logic_vector(31 downto 0);
	signal gpio1_clkout_tmp	: std_logic_vector(1 downto 0);

begin

-- clock generate
	rstn <= BUTTON(0);

	U_CLKGEN : CLKGEN
	port map (
		CLK50M		=> CLK50M0,
		RSTN		=> rstn,
		CLK108MOUT	=> clk108m,
		LOCK_PLL	=> lock_pll
	);


-- clock detect and data latch

	p6_rgbh <= p6_h & p6_g & p6_r & p6_b;

	U_P6SYNC : P6SYNC
	port map (
		P6_HSYNCN	=> p6_hsyncn,
		P6_VSYNCN	=> p6_vsyncn,
		P6_RGBH		=> p6_rgbh,
		P6_CLK		=> p6_clk,
		CLK108M		=> clk108m,
		RSTN		=> rstn,
		SRMODE		=> sy_srmode,
		CLKENB		=> sy_clkenb,
		DT_HSYNCN	=> sy_hsyncn,
		DT_VSYNCN	=> sy_vsyncn,
		DT_RGBH		=> sy_rgbh,
		SFTDT_U		=> sy_sftdt_u,
		SFTDT_D		=> sy_sftdt_d,
		SFTHS_U		=> sy_sfths_u,
		SFTHS_D		=> sy_sfths_d,
		LOCK1		=> sy_lock1,
		LOCK2		=> sy_lock2
	);

-- data shift
	U_DATASFT : DATASFT
	port map (
		HSYNCN		=> sy_hsyncn,
		VSYNCN		=> sy_vsyncn,
		RGBH		=> sy_rgbh,
		CLKENB		=> sy_clkenb,
		SRMODE		=> sy_srmode,
		LOCK1		=> sy_lock1,
		LOCK2		=> sy_lock2,
		CLK108M		=> clk108m,
		RSTN		=> rstn,
		HSYNCN_O	=> dt_hsyncn,
		VSYNCN_O	=> dt_vsyncn,
		RGBH_O		=> dt_rgbh,
		LOCK3		=> dt_lock3,
		LOCK4		=> dt_lock4
	);


-- VGA output
	U_VGAOUT : VGAOUT
	port map (
		DISPD		=> dt_rgbh,
		DISPTMG_HS	=> dt_hsyncn,
		DISPTMG_VS	=> dt_vsyncn,
		CLKENB		=> sy_clkenb,
		SRMODE		=> sy_srmode,
		SC4COLORON	=> colormd(2),
		SC4COLORMD	=> colormd(1 downto 0),
		LOCK_PLL	=> lock_pll,
		CLK108M		=> clk108m,
		RSTN		=> rstn,
		VGA_R		=> vga_r_i,
		VGA_G		=> vga_g_i,
		VGA_B		=> vga_b_i,
		VGA_HS		=> vga_hs_i,
		VGA_VS		=> vga_vs_i
	);

	VGA_R  <= vga_r_i;
	VGA_G  <= vga_g_i;
	VGA_B  <= vga_b_i;
	VGA_HS <= vga_hs_i;
	VGA_VS <= vga_vs_i;


-- color mode select

	mdsel <= not BUTTON(2);

	U_COLSEL : COLSEL
	port map (
		MDSEL		=> mdsel,
		LOCK_PLL	=> lock_pll,
		CLK108M		=> clk108m,
		RSTN		=> rstn,
		COLORMD		=> colormd
	);

-- 7seg displat

	U_SEG7DISP : SEG7DISP
	port map (
		SFTDT_U		=> sy_sftdt_u,
		SFTDT_D		=> sy_sftdt_d,
		SFTHS_U		=> sy_sfths_u,
		SFTHS_D		=> sy_sfths_d,
		HEX3_D		=> sg_hex3_d,
		HEX3_DP		=> sg_hex3_dp,
		HEX2_D		=> sg_hex2_d,
		HEX2_DP		=> sg_hex2_dp,
		HEX1_D		=> sg_hex1_d,
		HEX1_DP		=> sg_hex1_dp,
		HEX0_D		=> sg_hex0_d,
		HEX0_DP		=> sg_hex0_dp
	);

	HEX3_D		<= sg_hex3_d;
	HEX3_DP		<= sg_hex3_dp;
	HEX2_D		<= sg_hex2_d;
	HEX2_DP		<= sg_hex2_dp;
	HEX1_D		<= sg_hex1_d;
	HEX1_DP		<= sg_hex1_dp;
	HEX0_D		<= sg_hex0_d;
	HEX0_DP		<= sg_hex0_dp;


	gpio0_d_tmp	     <= (others => '0');
	gpio0_clkout_tmp <= (others => '0');

	GPIO0_CLKOUT(1 downto 0)	<= gpio0_clkout_tmp;

	GPIO0_D(31 downto 12) <= gpio0_d_tmp(31 downto 12);
	GPIO0_D( 7 downto  5) <= gpio0_d_tmp( 7 downto  5);
	GPIO0_D( 1 downto  0) <= gpio0_d_tmp( 1 downto  0);


	p6_h      <= GPIO0_D(11);
	p6_b      <= GPIO0_D(10);
	p6_g      <= GPIO0_D(9);
	p6_r      <= GPIO0_D(8);
	p6_hsyncn <= GPIO0_D(4);
	p6_vsyncn <= GPIO0_D(3);
	p6_clk    <= GPIO0_D(2);


	LEDG(0) <= colormd(2);
	LEDG(1) <= sy_srmode;
	LEDG(2) <= lock_pll;
	LEDG(3) <= sy_lock1;
	LEDG(4) <= sy_lock2;
	LEDG(5) <= dt_lock3;
	LEDG(6) <= dt_lock4;
	LEDG(7) <= '0';
	LEDG(8) <= '0';
	LEDG(9) <= '0';

-- probe output

	probdat1 <= (others => '0');
	probdat2 <= (others => '0');
	probdat3 <= (others => '0');

	gpio1_d_tmp	     <= (others => '0');
	gpio1_clkout_tmp <= (others => '0');

	GPIO1_D(0)		<= 'Z';
	GPIO1_D(1)		<= 'Z';

	GPIO1_D(3)		<= gpio1_d_tmp(3)      when (SW(9) = '0') else probdat1(0)	;	-- PROBE #00(trigger)
	GPIO1_D(2)		<= gpio1_d_tmp(2)      when (SW(9) = '0') else probdat1(1)	;	-- PROBE #01(trigger)
	GPIO1_D(5)		<= gpio1_d_tmp(5)      when (SW(9) = '0') else probdat1(2)	;	-- PROBE #02(trigger)
	GPIO1_D(4)		<= gpio1_d_tmp(4)      when (SW(9) = '0') else probdat1(3)	;	-- PROBE #03(trigger)
	GPIO1_D(7)		<= gpio1_d_tmp(7)      when (SW(9) = '0') else probdat1(4)	;	-- PROBE #04(trigger)
	GPIO1_D(6)		<= gpio1_d_tmp(6)      when (SW(9) = '0') else probdat1(5)	;	-- PROBE #05(trigger)
	GPIO1_D(9)		<= gpio1_d_tmp(9)      when (SW(9) = '0') else probdat1(6)	;	-- PROBE #06(trigger)
	GPIO1_D(8)		<= gpio1_d_tmp(8)      when (SW(9) = '0') else probdat1(7)	;	-- PROBE #07(trigger)
	GPIO1_D(11)		<= gpio1_d_tmp(11)     when (SW(9) = '0') else probdat2(0)	;	-- PROBE #08(trigger)
	GPIO1_D(10)		<= gpio1_d_tmp(10)     when (SW(9) = '0') else probdat2(1)	;	-- PROBE #09(trigger)
	GPIO1_D(13)		<= gpio1_d_tmp(13)     when (SW(9) = '0') else probdat2(2)	;	-- PROBE #10(trigger)
	GPIO1_D(12)		<= gpio1_d_tmp(12)     when (SW(9) = '0') else probdat2(3)	;	-- PROBE #11(trigger)
	GPIO1_D(14)		<= gpio1_d_tmp(14)     when (SW(9) = '0') else probdat2(4)	;	-- PROBE #12(trigger)
	GPIO1_CLKOUT(0)	<= gpio1_clkout_tmp(0) when (SW(9) = '0') else probdat2(5)	;	-- PROBE #13(trigger)
	GPIO1_D(15)		<= gpio1_d_tmp(15)     when (SW(9) = '0') else probdat2(6)	;	-- PROBE #14(trigger)
	GPIO1_CLKOUT(1)	<= gpio1_clkout_tmp(1) when (SW(9) = '0') else probdat2(7)	;	-- PROBE #15
	GPIO1_D(17)		<= gpio1_d_tmp(17)     when (SW(9) = '0') else probdat2(8)	;	-- PROBE #16
	GPIO1_D(16)		<= gpio1_d_tmp(16)     when (SW(9) = '0') else probdat2(9)	;	-- PROBE #17
	GPIO1_D(19)		<= gpio1_d_tmp(19)     when (SW(9) = '0') else probdat2(10)	;	-- PROBE #18
	GPIO1_D(18)		<= gpio1_d_tmp(18)     when (SW(9) = '0') else probdat2(11)	;	-- PROBE #19
	GPIO1_D(21)		<= gpio1_d_tmp(21)     when (SW(9) = '0') else probdat2(12)	;	-- PROBE #20
	GPIO1_D(20)		<= gpio1_d_tmp(20)     when (SW(9) = '0') else probdat2(13)	;	-- PROBE #21
	GPIO1_D(23)		<= gpio1_d_tmp(23)     when (SW(9) = '0') else probdat2(14)	;	-- PROBE #22
	GPIO1_D(22)		<= gpio1_d_tmp(22)     when (SW(9) = '0') else probdat2(15)	;	-- PROBE #23
	GPIO1_D(25)		<= gpio1_d_tmp(25)     when (SW(9) = '0') else probdat3(0)	;	-- PROBE #24
	GPIO1_D(24)		<= gpio1_d_tmp(24)     when (SW(9) = '0') else probdat3(1)	;	-- PROBE #25
	GPIO1_D(27)		<= gpio1_d_tmp(27)     when (SW(9) = '0') else probdat3(2)	;	-- PROBE #26
	GPIO1_D(26)		<= gpio1_d_tmp(26)     when (SW(9) = '0') else probdat3(3)	;	-- PROBE #27
	GPIO1_D(29)		<= gpio1_d_tmp(29)     when (SW(9) = '0') else probdat3(4)	;	-- PROBE #28
	GPIO1_D(28)		<= gpio1_d_tmp(28)     when (SW(9) = '0') else probdat3(5)	;	-- PROBE #29
	GPIO1_D(31)		<= gpio1_d_tmp(31)     when (SW(9) = '0') else probdat3(6)	;	-- PROBE #30
	GPIO1_D(30)		<= gpio1_d_tmp(30)     when (SW(9) = '0') else probdat3(7)	;	-- PROBE #31


-- unused pin
	UART_TXD		<= '0';
	UART_CTS		<= '0';
	DRAM_A			<= (others => '0');
	DRAM_LDQM		<= '0';
	DRAM_UDQM		<= '0';
	DRAM_WE_N		<= '1';
	DRAM_CAS_N		<= '1';
	DRAM_RAS_N		<= '1';
	DRAM_CS_N		<= '1';
	DRAM_BA_0		<= '0';
	DRAM_BA_1		<= '0';
	DRAM_CLK		<= '0';
	DRAM_CKE		<= '0';
	FLASH_A			<= (others => '0');
	FLASH_WE_N		<= '1';
	FLASH_RESET_N	<= '0';
	FLASH_OE_N		<= '1';
	FLASH_CE_N		<= '1';
	FLASH_WP_N		<= '1';
	FLASH_BYTE_N	<= '1';
	LCD_BLON		<= '0';
	LCD_RW			<= '0';
	LCD_EN			<= '0';
	LCD_RS			<= '0';
	SD_CLK			<= '0';
	GPIO0_CLKOUT	<= (others => '0');

	DRAM_D			<= (others => 'Z');
	FLASH_D15_AM1	<= 'Z';
	FLASH_D			<= (others => 'Z');
	LCD_D			<= (others => 'Z');
	PS2_KBDAT		<= 'Z';
	PS2_KBCLK		<= 'Z';
	PS2_MSDAT		<= 'Z';
	PS2_MSCLK		<= 'Z';
	SD_DAT			<= (others => 'Z');
	SD_CMD			<= 'Z';

end RTL;
